@@ -11,7 +11,7 @@
 __docformat__ = 'restructuredtext'
 
 import os
-from os.path import exists, join as opj
+from os.path import join as opj
 from collections import OrderedDict
 from operator import itemgetter
 
@@ -24,8 +24,10 @@ from ..cmd import link_file_load
 from ..support.archives import ArchivesCache
 from ..support.network import URL
 from ..support.locking import lock_if_check_fails
+from ..support.path import exists
 from ..utils import getpwd
 from ..utils import unique
+from ..utils import assure_bytes
 from .base import AnnexCustomRemote
 from .main import main as super_main
 
@@ -225,6 +227,7 @@ class ArchiveAnnexCustomRemote(AnnexCustomRemote):
                 # if for testing we want to force getting the archive extracted
                 # _ = self.cache.assure_extracted(self._get_key_path(akey)) # TEMP
                 efile = self.cache[akey_path].get_extracted_filename(afile)
+                efile = assure_bytes(efile)
 
                 if exists(efile):
                     size = os.stat(efile).st_size
@@ -357,7 +360,7 @@ class ArchiveAnnexCustomRemote(AnnexCustomRemote):
                 #  https://github.com/wummel/patool/issues/20
                 # so
                 pwd = getpwd()
-                lgr.debug("Getting file {afile} from {akey_path} while PWD={pwd}".format(**locals()))
+                lgr.debug(u"Getting file {afile} from {akey_path} while PWD={pwd}".format(**locals()))
                 apath = self.cache[akey_path].get_extracted_file(afile)
                 link_file_load(apath, path)
                 self.send('TRANSFER-SUCCESS', cmd, key)
