@@ -39,6 +39,7 @@ from datalad.support.network import RI
 from datalad.utils import getpwd
 from datalad.utils import optional_args, expandpath, is_explicit_path
 from datalad.utils import get_dataset_root
+from datalad.utils import dlabspath
 from datalad.distribution.utils import get_git_dir
 
 
@@ -62,10 +63,16 @@ def resolve_path(path, ds=None):
     -------
     Absolute path
     """
+    # first make sure it's actually a valid path:
+    from datalad.support.network import PathRI
+    if not isinstance(RI(path), PathRI):
+        raise ValueError("%s is not a valid path" % path)
+
     path = expandpath(path, force_absolute=False)
-    # TODO: normpath?!
     if is_explicit_path(path):
-        return abspath(path)
+        # normalize path consistently between two (explicit and implicit) cases
+        return dlabspath(path, norm=True)
+
     # no dataset given, use CWD as reference
     # note: abspath would disregard symlink in CWD
     top_path = getpwd() \
@@ -265,7 +272,7 @@ class Dataset(object):
         # with it. Internally we don't need or use it anymore.
         import inspect
         lgr.warning('%s still uses Dataset.get_subdatasets(). RF to use `subdatasets` command', inspect.stack()[1][3])
-        from datalad.api import subdatasets
+        from datalad.coreapi import subdatasets
         if edges:
             return [(r['parentpath'] if absolute else relpath(r['parentpath'], start=self.path),
                      r['path'] if absolute else relpath(r['path'], start=self.path))
@@ -324,7 +331,7 @@ class Dataset(object):
             return was_once_installed
 
     def get_superdataset(self, datalad_only=False, topmost=False,
-                         registered_only=False):
+                         registered_only=True):
         """Get the dataset's superdataset
 
         Parameters
@@ -343,6 +350,7 @@ class Dataset(object):
         -------
         Dataset or None
         """
+        from datalad.coreapi import subdatasets
         # TODO: return only if self is subdataset of the superdataset
         #       (meaning: registered as submodule)?
         path = self.path
@@ -359,14 +367,13 @@ class Dataset(object):
             sds = Dataset(sds_path_)
             if datalad_only:
                 # test if current git is actually a dataset?
-                # can't use ATM since we just autogenerate and ID, see
-                # https://github.com/datalad/datalad/issues/986
-                # if not sds.id:
-                if not sds.config.get('datalad.dataset.id', None):
+                if not sds.id:
                     break
             if registered_only:
                 if path not in sds.subdatasets(
-                        recursive=False, result_xfm='paths'):
+                        recursive=False,
+                        contains=path,
+                        result_xfm='paths'):
                     break
 
             # That was a good candidate
@@ -503,3 +510,6 @@ def require_dataset(dataset, check_installed=True, purpose=None):
                          "{0}.".format(dataset.path))
 
     return dataset
+
+
+lgr.log(5, "Done importing dataset")
\ No newline at end of file
