@@ -21,9 +21,11 @@ from datalad.utils import swallow_outputs
 from datalad.tests.utils import assert_in
 from datalad.tests.utils import assert_result_count
 from datalad.tests.utils import assert_is_generator
+from datalad.tests.utils import known_failure_direct_mode
 from datalad.tests.utils import with_tempfile
 from datalad.tests.utils import with_testsui
 from datalad.tests.utils import ok_clean_git
+from datalad.tests.utils import ok_file_under_git
 from datalad.tests.utils import SkipTest
 from datalad.tests.utils import eq_
 from datalad.support.exceptions import NoDatasetArgumentFound
@@ -180,6 +182,11 @@ def test_within_ds_file_search(path):
     except ImportError:
         raise SkipTest
     ds = Dataset(path).create(force=True)
+    # override default and search for datasets and files for this test
+    for m in ('egrep', 'textblob', 'autofield'):
+        ds.config.add(
+            'datalad.search.index-{}-documenttype'.format(m), 'all',
+            where='dataset')
     ds.config.add('datalad.metadata.nativetype', 'audio', where='dataset')
     makedirs(opj(path, 'stim'))
     for src, dst in (
@@ -188,6 +195,15 @@ def test_within_ds_file_search(path):
             opj(dirname(dirname(__file__)), 'tests', 'data', src),
             opj(path, dst))
     ds.add('.')
+    # yoh: CANNOT FIGURE IT OUT since in direct mode it gets added to git
+    # directly BUT
+    #  - output reports key, so seems to be added to annex!
+    #  - when I do manually in cmdline - goes to annex
+    ok_file_under_git(path, opj('stim', 'stim1.mp3'), annexed=True)
+    # If it is not under annex, below addition of metadata silently does
+    # not do anything
+    list(ds.repo.set_metadata(
+        opj('stim', 'stim1.mp3'), init={'importance': 'very'}))
     ds.aggregate_metadata()
     ok_clean_git(ds.path)
     # basic sanity check on the metadata structure of the dataset
@@ -213,6 +229,7 @@ type
 """, cmo.out)
 
     target_out = """\
+annex.importance
 audio.bitrate
 audio.duration(s)
 audio.format
@@ -239,49 +256,72 @@ type
 
     assert_result_count(ds.search('blablob#'), 0)
     # now check that we can discover things from the aggregated metadata
-    for mode, query, hitpath, matched_key, matched_val in (
-            # random keyword query
-            ('textblob',
-             'mp3',
-             opj('stim', 'stim1.mp3'),
-             'meta', 'mp3'),
-            # report which field matched with auto-field
-            ('autofield',
-             'mp3',
-             opj('stim', 'stim1.mp3'),
-             'audio.format', 'mp3'),
-            # XXX next one is not supported by current text field analyser
-            # decomposes the mime type in [mime, audio, mp3]
-            # ('autofield',
-            # "'mime:audio/mp3'",
-            # opj('stim', 'stim1.mp3'),
-            # 'audio.format', 'mime:audio/mp3'),
-            # but this one works
-            ('autofield',
-             "'mime audio mp3'",
-             opj('stim', 'stim1.mp3'),
-             'audio.format', 'mp3'),
-            # TODO extend with more complex queries to test whoosh
-            # query language configuration
+    for mode, query, hitpath, matched in (
+        ('egrep',
+         ':mp3',
+         opj('stim', 'stim1.mp3'),
+         {'audio.format': 'mp3'}),
+        # same as above, leading : is stripped, in indicates "ALL FIELDS"
+        ('egrep',
+         'mp3',
+         opj('stim', 'stim1.mp3'),
+         {'audio.format': 'mp3'}),
+        # same as above, but with AND condition
+        # get both matches
+        ('egrep',
+         ['mp3', 'type:file'],
+         opj('stim', 'stim1.mp3'),
+         {'type': 'file', 'audio.format': 'mp3'}),
+        # case insensitive search
+        ('egrep',
+         'mp3',
+         opj('stim', 'stim1.mp3'),
+         {'audio.format': 'mp3'}),
+        # field selection by expression
+        ('egrep',
+         'audio\.+:mp3',
+         opj('stim', 'stim1.mp3'),
+         {'audio.format': 'mp3'}),
+        # random keyword query
+        ('textblob',
+         'mp3',
+         opj('stim', 'stim1.mp3'),
+         {'meta': 'mp3'}),
+        # report which field matched with auto-field
+        ('autofield',
+         'mp3',
+         opj('stim', 'stim1.mp3'),
+         {'audio.format': 'mp3'}),
+        # XXX next one is not supported by current text field analyser
+        # decomposes the mime type in [mime, audio, mp3]
+        # ('autofield',
+        # "'mime:audio/mp3'",
+        # opj('stim', 'stim1.mp3'),
+        # 'audio.format', 'mime:audio/mp3'),
+        # but this one works
+        ('autofield',
+         "'mime audio mp3'",
+         opj('stim', 'stim1.mp3'),
+         {'audio.format': 'mp3'}),
+        # TODO extend with more complex queries to test whoosh
+        # query language configuration
     ):
         res = ds.search(query, mode=mode, full_record=True)
-        if mode == 'textblob':
-            # 'textblob' does datasets by default only (be could be configured otherwise
-            assert_result_count(res, 1)
-        else:
-            # the rest has always a file and the dataset, because they carry metadata in
-            # the same structure
-            assert_result_count(res, 2)
-            assert_result_count(
-                res, 1, type='file', path=opj(ds.path, hitpath),
-                # each file must report the ID of the dataset it is from, critical for
-                # discovering related content
-                dsid=ds.id)
         assert_result_count(
-            res, 1, type='dataset', path=ds.path, dsid=ds.id)
+            res, 1, type='file', path=opj(ds.path, hitpath),
+            # each file must report the ID of the dataset it is from, critical for
+            # discovering related content
+            dsid=ds.id)
+        # in egrep we currently do not search unique values
+        # and the queries above aim at files
+        assert_result_count(res, 1 if mode == 'egrep' else 2)
+        if mode != 'egrep':
+            assert_result_count(
+                res, 1, type='dataset', path=ds.path, dsid=ds.id)
         # test the key and specific value of the match
-        assert_in(matched_key, res[-1]['query_matched'])
-        assert_equal(res[-1]['query_matched'][matched_key], matched_val)
+        for matched_key, matched_val in matched.items():
+            assert_in(matched_key, res[-1]['query_matched'])
+            assert_equal(res[-1]['query_matched'][matched_key], matched_val)
 
 
 def test_listdict2dictlist():
