@@ -8,6 +8,7 @@
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 
 import os
+import os.path as op
 
 from .utils import (
     chpwd,
@@ -18,12 +19,31 @@ from .utils import (
     assert_raises,
     assert_equal,
     assert_in,
+    ok_file_has_content,
 )
 from datalad.support.gitrepo import check_git_configured
 
 from mock import patch
 
 
+# verify that any target platform can deal with forward slashes
+# as os.path.sep, regardless of its native preferences
+@with_tree(tree={'subdir': {'testfile': 'testcontent'}})
+def test_paths_with_forward_slashes(path):
+    # access file with native absolute path spec
+    print(path)
+    ok_file_has_content(op.join(path, 'subdir', 'testfile'), 'testcontent')
+    with chpwd(path):
+        # native relative path spec
+        ok_file_has_content(op.join('subdir', 'testfile'), 'testcontent')
+        # posix relative path spec
+        ok_file_has_content('subdir/testfile', 'testcontent')
+    # abspath with forward slash path sep char
+    ok_file_has_content(
+        op.join(path, 'subdir', 'testfile').replace(op.sep, '/'),
+        'testcontent')
+
+
 #@with_tempfile(mkdir=True)
 # with_tempfile dereferences tempdir, so does not trigger the failure
 # on Yarik's laptop where TMPDIR=~/.tmp and ~/.tmp -> /tmp.
@@ -57,6 +77,13 @@ def test_git_config_fixture():
         assert_equal(sorted(check_git_configured()), ['user.email', 'user.name'])
 
 
+def test_no_empty_http_proxy():
+    # in __init__ we might prune http_proxy if it is empty, so it must not be
+    # empty if present
+    assert os.environ.get('http_proxy', 'default')
+    assert os.environ.get('https_proxy', 'default')
+
+
 @with_tree(tree={})
 def test_git_config_warning(path):
     with chpwd(path), \
@@ -65,4 +92,3 @@ def test_git_config_warning(path):
         # no configs in that empty HOME
         assert_equal(check_git_configured(), {})
         assert_in("configure git first", cml.out)
-
